#!/usr/bin/env ruby

def is_numeric(s)
	Float(s) != nil rescue false
end

puts "image_mount v1.0 by Robin Wood - robin@digininja.org - www.digininja.org"
puts

if ARGV[0] == "" or ARGV[0].nil? or ARGV[0] == "-h" or ARGV[0] == "--help" or ARGV[0] == "-?"
	puts "Usage:"
	puts "image_mount <filename> - to show available partitions"
	puts "image_mount <filename> <partition number> - to give the mount command to mount that partition"
	puts "image_mount <filename> <partition number> <mount point> - mount the partition on the specified mount point"
	puts
	exit
end

partition = nil
if !ARGV[1].nil?
	if is_numeric(ARGV[1])
		partition = ARGV[1].to_i
	else
		puts "Non-numeric partition number entered"
		puts
		exit
	end
end

filename=ARGV[0]

if !File.exists?(filename)
	puts "Image not found"
	puts
	exit
end

mount_point = nil

if !ARGV[2].nil?
	mount_point = ARGV[2]
	if !File.exists?(mount_point)
		puts "Mount point not found"
		puts
		exit
	end
end

cmd="/sbin/fdisk -ul #{filename} 2>/dev/null"

fdisk_out = %x{#{cmd}}

partitions = []

if partition.nil?
	puts "Partitions found:"
end

sector_size = nil

fdisk_out.each { |line|
	if /Units = sectors of 1 \* 512 = ([0-9]*) bytes/.match(line)
		sector_size = $1.to_i
		#puts "Sector Size = " + sector_size.to_s
	end
	if /#{filename}([0-9]*)\s+\*?\s*([0-9]+)\s*[0-9]+\s*[0-9]+\s*[0-9a-z]+\s*(.*)/i.match(line)
		partition_number = $1.to_i
		start = $2.to_i

		if $3 != ""
			partition_type = $3
		else
			partition_type = "Unknown"
		end

		partitions[partition_number] = start

		if partition.nil?
			puts "partition " + partition_number.to_s + " of type " + partition_type + " starts at " + start.to_s
		end
	end
}

if partition.nil?
	puts
	exit
end

if partitions[partition].nil?
	puts "Specified partition not found"
	puts
	exit
end
 
offset = sector_size * partitions[partition]
if !mount_point.nil?
	puts "Mounting partition " + partition.to_s + " from file " + filename + " to " + mount_point
	puts
	command = "mount -o ro,loop,offset=" + offset.to_s + " -t auto " + filename + " " + mount_point
	system (command)
	exit
end

puts "mount -o ro,loop,offset=" + offset.to_s + " -t auto " + filename + " /media/image"
puts
